@@ -9,11 +9,27 @@ package Sidef::Types::Array::Matrix {
 
     require List::Util;
 
-    my %array_like = (
-                      'Sidef::Types::Array::Array'  => 1,
-                      'Sidef::Types::Array::Matrix' => 1,
-                      'Sidef::Types::Array::Pair'   => 1,
-                     );
+    sub _is_matrix {
+        my ($self) = @_;
+
+        my $ref = ref($self);
+
+        if ($ref eq __PACKAGE__) {
+            return 1;
+        }
+
+        if ($ref and UNIVERSAL::isa($self, 'Sidef::Types::Array::Array')) {
+
+            foreach my $row (@$self) {    # each row must an array-like object
+                ref($row) and UNIVERSAL::isa($row, 'Sidef::Types::Array::Array')
+                  or return 0;
+            }
+
+            return 1;
+        }
+
+        return 0;
+    }
 
     sub new {
         my (undef, @rows) = @_;
@@ -260,7 +276,7 @@ package Sidef::Types::Array::Matrix {
     sub add {
         my ($m1, $m2) = @_;
 
-        if (exists $array_like{ref($m2)}) {
+        if (_is_matrix($m2)) {
             return bless($m1->wise_operator('+', $m2));
         }
 
@@ -270,7 +286,7 @@ package Sidef::Types::Array::Matrix {
     sub sub {
         my ($m1, $m2) = @_;
 
-        if (exists $array_like{ref($m2)}) {
+        if (_is_matrix($m2)) {
             return bless($m1->wise_operator('-', $m2));
         }
 
@@ -280,7 +296,7 @@ package Sidef::Types::Array::Matrix {
     sub div {
         my ($m1, $m2) = @_;
 
-        if (exists $array_like{ref($m2)}) {
+        if (_is_matrix($m2)) {
             return $m1->mul($m2->inv);
         }
 
@@ -290,7 +306,7 @@ package Sidef::Types::Array::Matrix {
     sub mul {
         my ($m1, $m2) = @_;
 
-        if (not exists $array_like{ref($m2)}) {
+        if (!_is_matrix($m2)) {
             return bless($m1->scalar_operator('*', $m2));
         }
 
@@ -325,28 +341,18 @@ package Sidef::Types::Array::Matrix {
 
     sub floor {
         my ($self) = @_;
-
-        bless [
-            map {
-                bless [map { $_->floor } @$_], 'Sidef::Types::Array::Array'
-              } @$self
-        ];
+        bless($self->scalar_operator('floor'));
     }
 
     sub ceil {
         my ($self) = @_;
-
-        bless [
-            map {
-                bless [map { $_->ceil } @$_], 'Sidef::Types::Array::Array'
-              } @$self
-        ];
+        bless($self->scalar_operator('ceil'));
     }
 
     sub mod {
         my ($m1, $m2) = @_;
 
-        if (exists $array_like{ref($m2)}) {
+        if (_is_matrix($m2)) {
             return $m1->sub($m2->mul($m1->div($m2)->floor));
         }
 
@@ -356,7 +362,7 @@ package Sidef::Types::Array::Matrix {
     sub and {
         my ($m1, $m2) = @_;
 
-        if (exists $array_like{ref($m2)}) {
+        if (_is_matrix($m2)) {
             return bless($m1->wise_operator('&', $m2));
         }
 
@@ -366,7 +372,7 @@ package Sidef::Types::Array::Matrix {
     sub or {
         my ($m1, $m2) = @_;
 
-        if (exists $array_like{ref($m2)}) {
+        if (_is_matrix($m2)) {
             return bless($m1->wise_operator('|', $m2));
         }
 
@@ -376,7 +382,7 @@ package Sidef::Types::Array::Matrix {
     sub xor {
         my ($m1, $m2) = @_;
 
-        if (exists $array_like{ref($m2)}) {
+        if (_is_matrix($m2)) {
             return bless($m1->wise_operator('^', $m2));
         }
 
@@ -719,7 +725,7 @@ package Sidef::Types::Array::Matrix {
     sub concat {
         my ($m1, $m2) = @_;
 
-        if (exists $array_like{ref($m2)}) {
+        if (_is_matrix($m2)) {
 
             my $end = List::Util::min($#{$m1}, $#{$m2});
 
