@@ -416,6 +416,8 @@ package Sidef::Types::Range::Range {
     sub to_array {
         my ($self) = @_;
 
+        ref($self) || return Sidef::Types::Array::Array->new($self);
+
         my @array;
         my $iter = $self->iter;
 
@@ -439,6 +441,8 @@ package Sidef::Types::Range::Range {
     sub to_list {
         my ($self) = @_;
 
+        ref($self) or return;
+
         my @array;
         my $iter = $self->iter;
 
@@ -644,6 +648,11 @@ package Sidef::Types::Range::Range {
         $self->new($self->{from}->div($arg), $self->{to}->div($arg), $self->{step});
     }
 
+    sub neg {
+        my ($self) = @_;
+        $self->new($self->{from}->neg, $self->{to}, $self->{step});
+    }
+
     sub eq {
         my ($r1, $r2) = @_;
 
